import * as ts from "typescript";
import { Logger } from "../utils/loggers";
import { Reflection, ProjectReflection, Type } from "../models/index";
import { Converter } from "./converter";
/**
 * The context describes the current state the converter is in.
 */
export declare class Context {
    /**
     * The converter instance that has created the context.
     */
    converter: Converter;
    /**
     * A list of all files that have been passed to the TypeScript compiler.
     */
    fileNames: readonly string[];
    /**
     * The TypeChecker instance returned by the TypeScript compiler.
     */
    checker: ts.TypeChecker;
    /**
     * The program that is currently processed.
     */
    program: ts.Program;
    /**
     * The project that is currently processed.
     */
    project: ProjectReflection;
    /**
     * The scope or parent reflection that is currently processed.
     */
    scope: Reflection;
    /**
     * Conversion is done in two passes to properly handle symbols exported from
     * more than one entry point. This is used by the SourceFile converter to check
     * if re-exports should be documented.
     */
    inFirstPass?: boolean;
    /**
     * Is the current source file marked as being external?
     */
    isExternal?: boolean;
    /**
     * Is the current source file a declaration file?
     */
    isDeclaration?: boolean;
    /**
     * The currently set type parameters.
     */
    typeParameters?: Record<string, Type | undefined>;
    /**
     * The currently set type arguments.
     */
    typeArguments?: Type[];
    /**
     * Is the converter in inheritance mode?
     */
    isInherit?: boolean;
    /**
     * The node that has started the inheritance mode.
     */
    inheritParent?: ts.Node;
    /**
     * List symbol fqns of inherited children already visited while inheriting.
     */
    inheritedChildren?: string[];
    /**
     * The names of the children of the scope before inheritance has been started.
     */
    inherited?: string[];
    /**
     * A list of parent nodes that have been passed to the visit function.
     */
    visitStack: ts.Node[];
    /**
     * The pattern that should be used to flag external source files.
     */
    private externalPattern?;
    /**
     * Create a new Context instance.
     *
     * @param converter  The converter instance that has created the context.
     * @param fileNames  A list of all files that have been passed to the TypeScript compiler.
     * @param checker  The TypeChecker instance returned by the TypeScript compiler.
     */
    constructor(converter: Converter, fileNames: readonly string[], checker: ts.TypeChecker, program: ts.Program);
    /** @internal */
    get logger(): Logger;
    /**
     * Return the compiler options.
     */
    getCompilerOptions(): ts.CompilerOptions;
    /**
     * Return the type declaration of the given node.
     *
     * @param node  The TypeScript node whose type should be resolved.
     * @returns The type declaration of the given node.
     */
    getTypeAtLocation(node: ts.Node): ts.Type | undefined;
    getSymbolAtLocation(node: ts.Node): ts.Symbol | undefined;
    expectSymbolAtLocation(node: ts.Node): ts.Symbol;
    resolveAliasedSymbol(symbol: ts.Symbol): ts.Symbol;
    resolveAliasedSymbol(symbol: ts.Symbol | undefined): ts.Symbol | undefined;
    /**
     * Return the current logger instance.
     *
     * @returns The current logger instance.
     */
    getLogger(): Logger;
    /**
     * Register a newly generated reflection. All created reflections should be
     * passed to this method to ensure that the project helper functions work correctly.
     *
     * @param reflection  The reflection that should be registered.
     * @param symbol  The symbol the given reflection was resolved from.
     */
    registerReflection(reflection: Reflection, symbol?: ts.Symbol): void;
    /**
     * Trigger a node reflection event.
     *
     * All events are dispatched on the current converter instance.
     *
     * @param name  The name of the event that should be triggered.
     * @param reflection  The triggering reflection.
     * @param node  The triggering TypeScript node if available.
     */
    trigger(name: string, reflection: Reflection, node?: ts.Node): void;
    /**
     * Run the given callback with the context configured for the given source file.
     *
     * @param node  The TypeScript node containing the source file declaration.
     * @param callback  The callback that should be executed.
     */
    withSourceFile(node: ts.SourceFile, callback: () => void): void;
    /**
     * @param callback  The callback function that should be executed with the changed context.
     */
    withScope(scope: Reflection | undefined, callback: () => void): void;
    /**
     * @param parameters  An array of type parameters that should be set on the context while the callback is invoked.
     * @param callback  The callback function that should be executed with the changed context.
     */
    withScope(scope: Reflection | undefined, parameters: ts.NodeArray<ts.TypeParameterDeclaration> | undefined, callback: () => void): void;
    /**
     * @param parameters  An array of type parameters that should be set on the context while the callback is invoked.
     * @param preserve  Should the currently set type parameters of the context be preserved?
     * @param callback  The callback function that should be executed with the changed context.
     */
    withScope(scope: Reflection | undefined, parameters: ts.NodeArray<ts.TypeParameterDeclaration> | undefined, preserve: boolean, callback: () => void): void;
    /**
     * Inherit the children of the given TypeScript node to the current scope.
     *
     * @param baseNode  The node whose children should be inherited.
     * @param typeArguments  The type arguments that apply while inheriting the given node.
     * @return The resulting reflection / the current scope.
     */
    inherit(baseNode: ts.Node, typeArguments?: ReadonlyArray<ts.TypeNode>): Reflection;
    /**
     * Determines if the given file is outside of the project's generated documentation.
     * This is not, and is not intended to be, foolproof. Plugins may remove reflections
     * in the file that this method returns true for, See GH#1166 for discussion on tradeoffs.
     *
     * @param fileName
     * @internal
     */
    isOutsideDocumentation(fileName: string, isExternal?: boolean): boolean;
    /**
     * Checks if the given file is external.
     *
     * @param fileName
     * @internal
     */
    isExternalFile(fileName: string): boolean;
    /**
     * Convert the given list of type parameter declarations into a type mapping.
     *
     * @param parameters  The list of type parameter declarations that should be converted.
     * @param preserve  Should the currently set type parameters of the context be preserved?
     * @returns The resulting type mapping.
     */
    private extractTypeParameters;
}
