"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createSignature = void 0;
const ts = require("typescript");
const index_1 = require("../../models/index");
const converter_1 = require("../converter");
const parameter_1 = require("./parameter");
/**
 * Create a new signature reflection from the given node.
 *
 * @param context  The context object describing the current state the converter is in.
 * @param node  The TypeScript node containing the signature declaration that should be reflected.
 * @param name  The name of the function or method this signature belongs to.
 * @param kind  The desired kind of the reflection.
 * @returns The newly created signature reflection describing the given node.
 */
function createSignature(context, node, name, kind) {
    const container = context.scope;
    if (!(container instanceof index_1.ContainerReflection)) {
        throw new Error("Expected container reflection.");
    }
    const signature = new index_1.SignatureReflection(name, kind, container);
    context.registerReflection(signature);
    context.withScope(signature, node.typeParameters, true, () => {
        node.parameters.forEach((parameter) => {
            parameter_1.createParameter(context, parameter);
        });
        signature.type = extractSignatureType(context, node);
        if (container.inheritedFrom) {
            signature.inheritedFrom = container.inheritedFrom.clone();
        }
    });
    context.trigger(converter_1.Converter.EVENT_CREATE_SIGNATURE, signature, node);
    return signature;
}
exports.createSignature = createSignature;
/**
 * Extract the return type of the given signature declaration.
 *
 * @param context  The context object describing the current state the converter is in.
 * @param node  The signature declaration whose return type should be determined.
 * @returns The return type reflection of the given signature.
 */
function extractSignatureType(context, node) {
    var _a;
    const checker = context.checker;
    if (node.kind & ts.SyntaxKind.CallSignature ||
        node.kind & ts.SyntaxKind.CallExpression) {
        try {
            const signature = checker.getSignatureFromDeclaration(node);
            // This is essentially what checker.getReturnTypeOfSignature will do, but doing it ourselves avoids type errors.
            if (!signature) {
                throw new Error("Failed to retrieve signature for node.");
            }
            return context.converter.convertType(context, (_a = node.type) !== null && _a !== void 0 ? _a : checker.getReturnTypeOfSignature(signature));
        }
        catch (error) {
            // ignore
        }
    }
    return context.converter.convertType(context, node.type);
}
//# sourceMappingURL=signature.js.map